#!/bin/bash

# Copyright (C) 2006-2018 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

#======================================================================================
# More common function from the network configuration scripts should moved to this file
#======================================================================================
CONFIGDIR="/etc/sysconfig/network-scripts/"
SYSCONFIG="/etc/sysconfig/"
infiniband_device() {
    local netdev="$1"
    hwtype="/sys/class/net/${netdev}/type"

    # hwtype will be 1 if Ethernet (/usr/include/net/if_arp.h)
    # hwtype will be 32 if InfiniBand (/usr/include/net/if_arp.h)
    # Return zero if hwtype is 32
    if [ "$(cat $hwtype 2>/dev/null)" = "32" ]; then
        return 0
    else
        return 1
    fi
}
bridge_port() {
    local port="$1"
    if /usr/bin/ovs-vsctl port-to-br $port > /dev/null 2>&1 ;then
        return 0
    else
        return 1
fi
}
clean_dhcp() {

 if [ -f "/var/run/dhclient-${1}.pid" ]; then
     dhcpid=`cat /var/run/dhclient-${1}.pid`
     /sbin/dhclient -r -lf /var/lib/dhclient/dhclient-${1}.leases -pf \
     /var/run/dhclient-${1}.pid ${1} >/dev/null 2>&1
     rm -f /var/run/dhclient-${1}.pid >/dev/null 2>&1
     kill $dhcpid >/dev/null 2>&1 || :
     if [ -f "/var/lib/dhclient/dhclient-${1}.leases" ]; then
         rm -f /var/lib/dhclient/dhclient-${1}.leases >/dev/null 2>&1
     fi
 fi
}
disable_lro()
{
    lro=`ethtool -k $1 | grep ^larg | awk -F: '{ print $2 }'`
    # is on in the string
    if [[ "${lro}" == *on* ]] ; then
        ethtool -K $1 lro off > /dev/null 2>&1 || true
        lro="off"
    fi
    dev_name=`ethtool -i $1 | grep driver | awk '{print $2}'`
    if [ "$dev_name" == "ixgbe" ]; then
        gro=`ethtool -k $1 | grep ^generic-receive | awk -F: '{ print $2 }'`
        if [[ "${gro}" == *on* ]] ; then
            ethtool -K $1 gro off > /dev/null 2>&1 || true
            gro="off"
        fi
    fi
    if [[ "${lro}" == 'off' ]] && [[ "${gro}" == 'off' ]]; then
       echo "ETHTOOL_OPTS=\"-K $1 lro off gro off\"" >> ${CONFIGDIR}ifcfg-$1
    elif [[ "${lro}" == 'off' ]]; then
       echo "ETHTOOL_OPTS=\"-K $1 lro off\"" >> ${CONFIGDIR}ifcfg-$1
    elif [[ "${gro}" == 'off' ]]; then
       echo "ETHTOOL_OPTS=\"-K $1 gro off\"" >> ${CONFIGDIR}ifcfg-$1
    fi

} 

enable_lro()
{
    lro=`ethtool -k $1 | grep ^larg | awk -F: '{ print $2 }'`
    # is fixed in the string
    if [[ "${lro}" == *fixed* ]] ; then
        return 0
    elif [ ${lro} == 'off' ] ; then
        ethtool -K $1 lro on > /dev/null 2>&1 || true
    fi
    sed -i '/ETHTOOL_OPTS/d' ${CONFIGDIR}ifcfg-$1 
    dev_name=`ethtool -i $1 | grep driver | awk '{print $2}'`
    if [ "$dev_name" == "ixgbe" ]; then
        ethtool -K $1 gro on > /dev/null 2>&1 || true
    fi
}

vxlan_interface()
{
   local netdev=$1
   devtype=`cat /sys/class/net/${netdev}/uevent | egrep DEVTYPE | awk -F= '{ print $2 }'`
   if [ "${devtype}" = 'vxlan' ]; then
      return 0
   else
      return 1
   fi
}

usb_interface()
{
   local netdev=$1
   sysfs_path="/sys/class/net/${netdev}/device/uevent"
   if [ -e ${sysfs_path} ]; then
      devtype=`cat ${sysfs_path} | egrep DEVTYPE | awk -F= '{ print $2 }'`
      if [ "${devtype}" = 'usb_interface' ]; then
         return 0
      fi
   fi
   return 1
}
