import logging
import time
import os

from agent.lib.daemon import init_log
from agent.lib.db import dump_db
from agent.lib.ocfs2 import (is_cluster_online, server_cluster_online,
                             server_cluster_offline, mount_device)
from agent.lib.settings import get_membership_state
from agent.lib.constants import (MEMBERSHIP_STATE_CLUSTERED, POOLFS_MOUNT,
                                 LOCAL_SERVER_DB)

from agent.lib.files import mount_repositories

MAX_RECOVERY_RUNS = 10
RECOVERY_INTERVAL = 60

def run_recovery():
    init_log()
    num_runs = MAX_RECOVERY_RUNS
    logging.debug("Recovery daemon started")

    server_db = dump_db(LOCAL_SERVER_DB)
    poolfs_target = server_db["poolfs_target"]
    poolfs_uuid = server_db["poolfs_uuid"]
    poolfs_mount_point = os.path.join(POOLFS_MOUNT, poolfs_uuid)

    logging.debug("target: %s, uuid: %s, mount: %s", poolfs_target,
                  poolfs_uuid, poolfs_mount_point)

    while num_runs:
        runs_str = "runs" if num_runs > 1 else "run"
        logging.debug("Recovery daemon: %s %s left", num_runs, runs_str)
        num_runs = num_runs - 1
        time.sleep(RECOVERY_INTERVAL)

        if get_membership_state() != MEMBERSHIP_STATE_CLUSTERED:
            try:
                server_cluster_offline()
            except:
                pass
            logging.debug("get_membership_state: not clustered - exiting")
            return
        else:
            if not os.path.exists(poolfs_target):
                logging.debug("Device %s not available", poolfs_target)
                continue
            else:
                logging.debug("trying to rejoin cluster")
                try:
                    if not is_cluster_online():
                        server_cluster_online()
                    else:
                        logging.debug("server already online")
                    if not os.path.ismount(poolfs_mount_point):
                        mount_device(poolfs_target, poolfs_mount_point, 5, 1)
                    else:
                        logging.debug("%s already mounted", poolfs_mount_point)
                    logging.debug("Success; remounting repositories")
                    mount_repositories()
                    break
                except:
                    logging.exception("unable to rejoin cluster")
                    continue

    logging.debug("Done")


if __name__ == '__main__':
    init_log()
    run_recovery()
