# Copyright (C) 2006-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import logging
import time
from agent.lib import filelock
from agent.lib.constants import LOCAL_SERVER_DB, CLUSTER_LOCK, SLEEP_ON_ERR
from agent.lib.config import get_config
from agent.lib.daemon import init_log
from agent.lib.db import read_item, write_item, dump_db, get_cluster_db_home
from agent.lib.settings import is_clustered
from agent.target.dlm import acquire_master_dlm_lock, release_master_dlm_lock


def update_pool_master_ip():
    """Update pool_master_ip field in server_pool db
    """
    try:
        d = dump_db(LOCAL_SERVER_DB)
        ip = d.get("registered_ip")
        write_item(LOCAL_SERVER_DB, "pool_master_ip", ip)
        db_home = get_cluster_db_home()
        write_item("server_pool", "pool_master_ip", ip, db_home)
    except Exception, e:
        logging.debug("Error in update_pool_master_ip: %s", e)


def teardown_master_env():
    """Teardown master environment:
       Release master dlm lock
    """
    release_master_dlm_lock()


take_crown = True


def cluster_remaster():
    global take_crown
    db_home = get_cluster_db_home()
    master_ip = read_item("server_pool", "pool_master_ip", db_home)

    if acquire_master_dlm_lock():
        if not master_ip:
            logging.debug("Master IP not present in server_pool DB")
            update_pool_master_ip()
            master_ip = read_item("server_pool", "pool_master_ip", db_home)

        if take_crown:
            take_crown = False
            write_item(LOCAL_SERVER_DB, "is_master", True)
            update_pool_master_ip()
            master_ip = read_item("server_pool", "pool_master_ip", db_home)
            logging.info("This server is the master, master IP=%s", master_ip)

    else:
        take_crown = True
        write_item(LOCAL_SERVER_DB, "is_master", False)


def remaster():
    lock = filelock.FileLock(CLUSTER_LOCK)
    lock.acquire(wait=60)
    try:
        if is_clustered():
            cluster_remaster()

    finally:
        lock.release()


def teardown_master_server():
    if is_clustered():
        teardown_master_env()

def serve_forever():
    logging.info("REMASTER SERVER STARTED")
    config = get_config()
    remaster_interval = int(config.get("remaster", "interval"))
    while True:
        try:
            time.sleep(remaster_interval)
            remaster()
        except Exception, e:
            logging.exception("Error in remaster process: %s", e)
            time.sleep(SLEEP_ON_ERR)


if __name__ == '__main__':
    init_log()
    serve_forever()
