#!/usr/bin/env python
#
# Copyright (C) 2011-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

"""Directives for managing the storage discovery process
"""

import os

from agent.storage.st_tech.multipath import get_multipath_conf
from agent.storage.st_discover.blkDevInfoUtils import (get_blk_dev_list,
                                        filter_blk_dev_list,
                                        filter_blk_dev_list_emc,
                                        populate_blk_dev_info_dict,
                                        populate_blk_dev_info_dict_emc,
                                        fill_blk_dev_info_dict____basic,
                                        fill_blk_dev_info_dict____multipath,
                                        fill_blk_dev_info_dict____emc)
from agent.storage.st_discover.storageXML import output_disk_XML
from agent.storage.st_xml.printXMLutils import (copy_XML_to_str, delete_XML)

__all__ = ['add_all_block_data']


def add_all_block_data(indent, item_LIST):

    (mpConf_default_callout, mpConf_blacklistInfo,
     mpConf_blacklistExceptionsInfo,
     mpConf_devSpec_LIST) = get_multipath_conf()
    blkdev_LIST = get_blk_dev_list()

    diskInfo_DICT = {}
    if not os.path.exists("/dev/emcpower"):
        blkdev_LIST = filter_blk_dev_list(blkdev_LIST,
                                          mpConf_blacklistInfo)
        populate_blk_dev_info_dict(diskInfo_DICT, blkdev_LIST,
                                   mpConf_default_callout,
                                   mpConf_blacklistInfo,
                                   mpConf_blacklistExceptionsInfo,
                                   mpConf_devSpec_LIST)
    else:
        find_List = ["sd"]
        blkdev_LIST = filter_blk_dev_list_emc(blkdev_LIST, find_List)
        populate_blk_dev_info_dict_emc(diskInfo_DICT, blkdev_LIST)

    fill_blk_dev_info_dict____basic(diskInfo_DICT)
    if not os.path.exists("/dev/emcpower"):
        fill_blk_dev_info_dict____multipath(diskInfo_DICT)
    else:
        fill_blk_dev_info_dict____emc(diskInfo_DICT)

    output_disk_XML(indent, 0, diskInfo_DICT, item_LIST)
    doc_str = copy_XML_to_str()
    delete_XML()

    return doc_str
