#!/usr/bin/env python
#
# Copyright (C) 2011-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

"""Utilities for dealing with QLogic HBA specifics
"""

import logging
import os
import re
import traceback

from agent.storage.st_utils.st_readAndWrite import read_text_lines

__all__ = ['get_qla4xxx_info_dict']


class QLogic_qla4xxx_State_OBJ:

    def __init__(self):
        self.mHBAnum = None
        self.mTargetnum = None
        self.mHWslot = None
        self.mTargetIP = None
        self.mTargetPort = -1
        self.mIQN = None
        self.mTpgt = -1

    def __repr__(self):
        return ("mHBAnum: %s mTargetnum: %s mHWslot: %s mTargetIP: %s "
                "mTargetPort: %s mIQN: %s, mTpgt: %d") % (self.mHBAnum,
                                                          self.mTargetnum,
                                                          self.mHWslot,
                                                          self.mTargetIP,
                                                          self.mTargetPort,
                                                          self.mIQN,
                                                          self.mTpgt)


def get_qla4xxx_info_dict(dummy_refresh):
    """
    QLogic_qla4xxx
    """
    if not os.path.exists('/proc/scsi/qla4xxx'):
        return {}
    qla4xxx_state_info_DICT = {}
    #scsi-qla0-target-1=ddb 0x0003:
    #   "iqn.2001-05.com.equallogic:6-8a0900-4bba77001-66800000075460c0-
    #    josh60gb1": 10.1.32.230;
    #scsi-qla0-target-2=ddb 0x0040: "iqn.2001-05.com.equallogic:6-8a0900-
    #    4d6a77001-ede00000079460c0-josh60gb2": 10.1.32.230;
    qla4xxx_state_path = ''
    begSect_cmpldRegEx = re.compile('^SCSI Device Information:$')
    endSect_cmpldRegEx = re.compile('^SCSI LUN Information:$')
    qla4xxxTrgt_cmpldRegEx = re.compile('^scsi-qla(\d+)-target-(\d+)='
                                        'ddb\s+(0x\d+):\s+"(\S*)":\s+(\d+'
                                        '\.\d+\.\d+\.\d+);$')
    for qla4xxx_state_filename in os.listdir('/proc/scsi/qla4xxx'):
        qla4xxx_state_path = "/proc/scsi/qla4xxx/%s" % (qla4xxx_state_filename)
        qla4xxx_state_lines, read_failed = read_text_lines(qla4xxx_state_path)
        if read_failed:
            logging.error("Problems extracting qla4xxx state file info %s",
                          qla4xxx_state_path)
            continue
        try:
            sectIdx = 0
            while not begSect_cmpldRegEx.match(qla4xxx_state_lines[sectIdx]):
                sectIdx = sectIdx + 1
            del(qla4xxx_state_lines[0:sectIdx + 1])
            for qla4xxx_state_line in qla4xxx_state_lines:
                if endSect_cmpldRegEx.match(qla4xxx_state_line):
                    break
                qla4xxxTrgt_match = qla4xxxTrgt_cmpldRegEx.match(
                                                            qla4xxx_state_line)
                if not qla4xxxTrgt_match:
                    continue
                if len(qla4xxxTrgt_match.group(4)) < 1:
                    continue
                qla4xxx_state = QLogic_qla4xxx_State_OBJ()
                qla4xxx_state.mHBAnum = int(qla4xxxTrgt_match.group(1))
                qla4xxx_state.mTargetnum = int(qla4xxxTrgt_match.group(2))
                qla4xxx_state.mHWslot = int(qla4xxxTrgt_match.group(3), 16)
                # This is the IP on OpeniSCSI so we match.
                qla4xxx_state.mTargetIP = qla4xxxTrgt_match.group(5)
                # Not there. Assuming default.
                qla4xxx_state.mTargetPort = 3260
                qla4xxx_state.mIQN = qla4xxxTrgt_match.group(4)
                # Not there either.  Use the mTargetnum
                qla4xxx_state.mTpgt = int(qla4xxx_state.mTargetnum)
                qla4xxx_state_info_DICT[
                                    qla4xxx_state.mTargetnum] = qla4xxx_state
            del(qla4xxx_state_lines)
        except StandardError:
            logging.error(
                "Unexpected qla4xxx state file format %s\n%s",
                qla4xxx_state_path, traceback.format_exc())
            del(qla4xxx_state_lines)
            continue

    return qla4xxx_state_info_DICT
