#!/usr/bin/env python
#
# Copyright (C) 2011-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

"""Utilities for dealing with generic devices
"""

import re
import logging

from agent.storage.st_utils.st_readAndWrite import read_text_lines

__all__ = ['get_block_device_major_num']


def get_device_major_nums():
    char_device_major_nums_LIST_DICT = {}
    blck_device_major_nums_LIST_DICT = {}
    proc_devices_lines_LIST, read_failed = read_text_lines('/proc/devices')
    if read_failed:
        logging.critical('read of /proc/devices failed !?!')
        return (None, None)
    maj_finder_cmpldRegEx = re.compile('\s*(\d+)\s+(\S+)\s*$')
    char_dev_start_marker_cmpldRegEx = re.compile('^Character devices:$')
    blck_dev_start_marker_cmpldRegEx = re.compile('^Block devices:$')
    idx = 0

    for idx in range(0, len(proc_devices_lines_LIST)):
        proc_devices_line = proc_devices_lines_LIST[idx]
        if not char_dev_start_marker_cmpldRegEx.match(proc_devices_line):
            continue
        idx += 1
        break

    for idx in range(idx, len(proc_devices_lines_LIST)):
        proc_devices_line = proc_devices_lines_LIST[idx]
        if not blck_dev_start_marker_cmpldRegEx.match(proc_devices_line):
            maj_finder_match = maj_finder_cmpldRegEx.match(proc_devices_line)
            if maj_finder_match:
                device_name = maj_finder_match.group(2)
                device_major_num = int(maj_finder_match.group(1))
                device_major_nums_LIST = char_device_major_nums_LIST_DICT.get(device_name, None)
                if not device_major_nums_LIST:
                    char_device_major_nums_LIST_DICT[device_name] = [device_major_num]
                else:
                    device_major_nums_LIST.append(device_major_num)
                continue
            if len(proc_devices_line) < 1:
                continue
            logging.error("BAD /proc/devices line '%s'", proc_devices_line)
            continue
        idx += 1
        break

    for idx in range(idx, len(proc_devices_lines_LIST)):
        proc_devices_line = proc_devices_lines_LIST[idx]
        maj_finder_match = maj_finder_cmpldRegEx.match(proc_devices_line)
        if maj_finder_match:
            device_name = maj_finder_match.group(2)
            device_major_num = int(maj_finder_match.group(1))
            device_major_nums_LIST = blck_device_major_nums_LIST_DICT.get(device_name, None)
            if not device_major_nums_LIST:
                blck_device_major_nums_LIST_DICT[device_name] = [device_major_num]
            else:
                device_major_nums_LIST.append(device_major_num)
            continue
        if len(proc_devices_line) < 1:
            continue
        logging.error("BAD /proc/devices line '%s'", proc_devices_line)

    return (char_device_major_nums_LIST_DICT,
            blck_device_major_nums_LIST_DICT)


def get_block_device_major_num(deviceName):
    (char_device_major_nums_LIST_DICT,
     blck_device_major_nums_LIST_DICT) = get_device_major_nums()
    if not char_device_major_nums_LIST_DICT:
        return -1
    device_major_nums_LIST = blck_device_major_nums_LIST_DICT.get(deviceName,
                                                                  None)
    if not device_major_nums_LIST:
        return -2
    if len(device_major_nums_LIST) != 1:
        return -3

    return device_major_nums_LIST[0]
