#!/usr/bin/env python
#
# Copyright (C) 2011-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

"""Utilities for helping with reading and writing
"""

import logging
import traceback

__all__ = ['read_text_value',
           'read_text_lines',
           'write_text_value']


def read_text_value(text_file_path):
    """
    This routine reads all of the lines of the text file at the given
    text_file_path and returns them as a single buffer

    @param       text_file_path        path to the text file to read

    NOTE:        lines are head tail whitespace stripped then
                 concatenated with a single space

    @return      read_buffer, did_read_fail_boolean
    """
    final_text_value = ''
    try:
        text_FILE = open(text_file_path, 'r')
        final_text_value = ' '.join(map(str.strip, text_FILE.readlines()))
        text_FILE.close()
    except IOError:
        return '', True
    except:
        logging.debug("extraction from %s failed\n%s",
                      text_file_path, traceback.format_exc())
        return '', True

    return final_text_value, False


def read_text_lines(text_file_path):
    """
    This routine reads all of the lines of the text file at
    the given text_file_path and returns them as a list of lines

    @param      text_file_path        path to the text file to read

    NOTE:       lines are head tail whitespace stripped

    @return     readLines_LIST, did_read_fail_boolean
    """
    text_lines_LIST = []
    try:
        text_FILE = open(text_file_path, 'r')
        text_lines_LIST = map(str.strip, text_FILE.readlines())
        text_FILE.close()
    except IOError:
        logging.debug("open( %s ) failed", text_file_path)
        return [], True
    except:
        logging.debug("extraction from %s failed\n%s",
                      text_file_path, traceback.format_exc())
        return [], True

    return text_lines_LIST, False


def write_text_value(text_file_path, buffer_to_write, appendVStruncate):
    """
    This routine writes the given buffer to the text file specified

    @param      text_file_path      path to the text file to write
                buffer_to_write     data to write
                appendVStruncate    boolean whether to append or to truncate

    @return     did_write_fail_boolean
    """
    if appendVStruncate:
        open_mode = 'a'
    else:
        open_mode = 'w'
    try:
        text_FILE = open(text_file_path, open_mode)
        text_FILE.write(buffer_to_write)
        text_FILE.close()
    except IOError:
        logging.error("open( %s ) failed", text_file_path)
        return True
    except:
        logging.error("write to %s failed\n%s",
                      text_file_path, traceback.format_exc())
        return True

    return False
