#!/usr/bin/env python
#
# Copyright (C) 2011-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

"""Utilities for working with DOM (Document Object Model)
"""

import xml.dom.minidom

__all__ = ['DOMdocHelper_OBJ']


class DOMdocHelper_OBJ:

    """Class for
    """
    def __init__(self, startingIndentXMLlvl, indentXMLlvlUnit):
        dom_impl = xml.dom.minidom.getDOMImplementation()
        self.mDoc = dom_impl.createDocument(None, "XML_DOC", None)
        self.mCurLvlDocElem = self.mDoc.documentElement
        self.mLvlDocElemStack_LIST = []
        self.mIndentXMLlvl = startingIndentXMLlvl  # in characters
        self.mIndentXMLlvlUnit = indentXMLlvlUnit  # in characters

    def destroy(self):  # MUST be called before del
        del self.mLvlDocElemStack_LIST
        self.mCurLvlDocElem = None
        self.mDoc.unlink()
        del self.mDoc

    def push_level(self, newLvlTag, attr_DICT=None):
        indent_text = self.mDoc.createTextNode('\n' + ' ' * self.mIndentXMLlvl)
        newLvlDocElem = self.mDoc.createElement(str(newLvlTag))
        if attr_DICT:
            for attrName, attrValueText in attr_DICT.iteritems():
                newLvlDocElem.setAttribute(str(attrName), str(attrValueText))
        self.mCurLvlDocElem.appendChild(indent_text)
        self.mCurLvlDocElem.appendChild(newLvlDocElem)
        self.mLvlDocElemStack_LIST.append(self.mCurLvlDocElem)
        self.mCurLvlDocElem = newLvlDocElem
        self.mIndentXMLlvl += self.mIndentXMLlvlUnit

    def add_data_elem(self, dataTag, dataValueText, attr_DICT=None):
        indent_text = self.mDoc.createTextNode('\n' + ' ' * self.mIndentXMLlvl)
        newDataElem = self.mDoc.createElement(str(dataTag))
        dataValueTextNode = self.mDoc.createTextNode(str(dataValueText))
        newDataElem.appendChild(dataValueTextNode)
        if attr_DICT:
            for attrName, attrValueText in attr_DICT.iteritems():
                newDataElem.setAttribute(str(attrName), str(attrValueText))
        self.mCurLvlDocElem.appendChild(indent_text)
        self.mCurLvlDocElem.appendChild(newDataElem)

    def pop_level(self):
        if len(self.mLvlDocElemStack_LIST) == 0:
            return 0
        self.mIndentXMLlvl -= self.mIndentXMLlvlUnit
        indent_text = self.mDoc.createTextNode('\n' + ' ' * self.mIndentXMLlvl)
        self.mCurLvlDocElem.appendChild(indent_text)
        self.mCurLvlDocElem = self.mLvlDocElemStack_LIST.pop()

        return len(self.mLvlDocElemStack_LIST)

    def finalize(self):
        while self.pop_level():
            pass
        indent_text = self.mDoc.createTextNode('\n' + ' ' * self.mIndentXMLlvl)
        self.mCurLvlDocElem.appendChild(indent_text)
        dom_doc_xml_lines_LIST = self.mDoc.toxml().split('\n')
        if dom_doc_xml_lines_LIST[0].find("xml version=") >= 0:
            dom_doc_xml_lines_LIST.pop(0)
        if dom_doc_xml_lines_LIST[0].find("XML_DOC") >= 0:
            dom_doc_xml_lines_LIST.pop(0)
        dom_doc_xml_lines_LIST.pop()
        self.destroy()    # Now unusable
        return dom_doc_xml_lines_LIST

    def __repr__(self):
        return "DOC: '%s'" % (self.mDoc.toxml())
