#!/usr/bin/env python
#
# Copyright (C) 2011-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

"""Utilities for accumulating and printing XML
"""

import os
import re
import string

from agent.storage.st_utils.st_readAndWrite import read_text_value

__all__ = ['print_beg_XML_tag',
           'print_XML',
           'print_XML_data',
           'print_end_XML_tag',
           'sys_item_XML',
           'copy_XML_to_str',
           'delete_XML']

global_XML_accumulation_LIST = []


def print_XML(indentXMLlvl, *itemsToPrint_TPL):
    global global_XML_accumulation_LIST

    if indentXMLlvl > 0:
        indentXMLlvl = indentXMLlvl - 1
        xmlBuffer = ' ' * indentXMLlvl
    else:
        xmlBuffer = ''
    for itemToPrint in itemsToPrint_TPL:
        if xmlBuffer:
            xmlBuffer = "%s %s" % (xmlBuffer, itemToPrint)
        else:
            xmlBuffer = str(itemToPrint)
    global_XML_accumulation_LIST.append(xmlBuffer)


def delete_XML():
    global global_XML_accumulation_LIST

    global_XML_accumulation_LIST = []


def copy_XML_to_str():
    global global_XML_accumulation_LIST

    XML_str = '\n'.join(global_XML_accumulation_LIST) + '\n'

    return XML_str


def sanitize_for_XML_tag(theXMLtag):
    if theXMLtag == None:
        return None
    sanitized_XMLtag = str(theXMLtag)
    sanitized_XMLtag = sanitized_XMLtag.replace('<', '_')
    sanitized_XMLtag = sanitized_XMLtag.replace('>', '_')

    return sanitized_XMLtag


def sanitize_for_XML_data(theXMLdata):
    if theXMLdata == None:
        return None
    sanitized_XMLdata = str(theXMLdata)
    sanitized_XMLdata = sanitized_XMLdata.replace('<', '_')
    sanitized_XMLdata = sanitized_XMLdata.replace('>', '_')

    return sanitized_XMLdata


def print_beg_XML_tag(indentXMLlvl, theXMLtag, theXMLdata, rawData=False):
    sanitized_XMLtag = sanitize_for_XML_tag(theXMLtag)
    if not rawData:
        sanitized_XMLdata = sanitize_for_XML_data(theXMLdata)
        if sanitized_XMLdata:
            print_XML(indentXMLlvl,
                      "<%s %s>" % (sanitized_XMLtag, sanitized_XMLdata))
        else:
            print_XML(indentXMLlvl, "<%s>" % (sanitized_XMLtag))
    else:
        # no need to sanitize data as it is in RAW XML tags
        if theXMLdata:
            print_XML(indentXMLlvl,
                      "<%s %s><![CDATA[" % (sanitized_XMLtag, theXMLdata))
        else:
            print_XML(indentXMLlvl, "<%s><![CDATA[" % (sanitized_XMLtag))


def print_end_XML_tag(indentXMLlvl, theXMLtag, rawData=False):
    sanitized_XMLtag = sanitize_for_XML_tag(theXMLtag)
    if not rawData:
        print_XML(indentXMLlvl, "</%s>" % (sanitized_XMLtag))
    else:
        print_XML(indentXMLlvl, "]]></%s>" % (sanitized_XMLtag))


def print_XML_data(indentXMLlvl, theXMLtag, theXMLdata):
    sanitized_XMLtag = sanitize_for_XML_tag(theXMLtag)
    sanitized_XMLdata = sanitize_for_XML_data(theXMLdata)
    print_XML(indentXMLlvl,
        "<%s>%s</%s>" %
        (sanitized_XMLtag, sanitized_XMLdata, sanitized_XMLtag))


def sys_item_XML(domHelper, sys_host_path, sys_item, exclude_LIST):
    sys_item_path = "%s/%s" % (sys_host_path, sys_item)
    if os.path.islink(sys_item_path):
        return

    for exclude_item in exclude_LIST:
        if re.match("^%s" % (exclude_item), sys_item):
#            Do not log excluded items (e.g. username, password)
            return
    nums = string.digits
    sys_item_tag = sys_item.title()
    sys_item_tag = sys_item_tag.replace(' ', '_')
    if sys_item_tag[0] in nums:
        sys_item_tag = 'X' + sys_item_tag
    sys_item_lines, read_failed = read_text_value(sys_item_path)
    if read_failed:
        return
    for idx in range(len(sys_item_lines)):
        if sys_item_lines[idx] not in string.printable:
            return
    domHelper.add_data_elem(sys_item_tag, sys_item_lines)
