# Copyright (C) 2006-2016 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import SimpleXMLRPCServer
import xmlrpclib
import sys


def fix_xmlrpclib():

    MAXINT = 2L**31-1
    MININT = -2L**31
    MAXLONG = 2L**63-1
    MINLONG = -2L**63

    def dump_long(self, value, write):
        if value > MAXLONG or value < MINLONG:
            raise OverflowError("long int exceeds XML-RPC limits")
        write("<value><i8>")
        write(str(long(value)))
        write("</i8></value>\n")

    def dump_int(self, value, write):
        # in case ints are > 32 bits
        if value > MAXINT or value < MININT:
            return dump_long(self, value, write)
        write("<value><int>")
        write(str(value))
        write("</int></value>\n")

    # Fix xmlrpclib.Marshaller.dispatch[types.LongType] and
    # xmlrpclib.Marshaller.dispatch[types.IntType] to the functions
    # that do what we want.
    import types
    xmlrpclib.Marshaller.dispatch[types.LongType] = dump_long
    xmlrpclib.Marshaller.dispatch[types.IntType] = dump_int


fix_xmlrpclib()

if sys.version_info >= (2, 7, 9):
    #
    # disable SSL certificate verification; it is default in python 2.7.9.
    # https://www.python.org/dev/peps/pep-0476/
    #
    # in ovs-agent all uses of xmlrpc are local to the host on which
    # ovs-agent runs, thus there is no need to vertify server's certificate.
    #
    import ssl
    ssl._create_default_https_context = ssl._create_unverified_context
